// part of the Irrlicht Engine Shader example.
// These simple Direct3D9 pixel and vertex shaders will be loaded by the shaders
// example. Please note that these example shaders don't do anything really useful.
// They only demonstrate that shaders can be used in Irrlicht.

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4x4 mWorldViewProj;  // World * View * Projection transformation
float4x4 mWorld;
float4x4 mInvWorld;       // Inverted world matrix
//float4x4 mTransWorld;     // Transposed world matrix
//float3 mLightPos;         // Light position
//float4 mLightColor;       // Light color

// soleil
float3 mSunDir;

// lumires
float3 mLight1Pos;
float3 mLight2Pos;
float3 mLight3Pos;
float3 mLight4Pos;
float3 mLight5Pos;

float2 mLight1Prop;
float2 mLight2Prop;
float2 mLight3Prop;
float2 mLight4Prop;
float2 mLight5Prop;

// lumire relle;
float3 mRealLight = (0, 0, 0);
float2 mRealLightProp = (10, 100);
float4 mRealLightColor = (0, 0, 0, 0);

// fog / brouillare
float2 fogDist  = (20, 200);
float3 camPos;


// ----------------------------------------------------------------------------------------

float calcDist(float3 v1, float3 v2)
{
    return sqrt(pow(v1.x - v2.x, 2) + pow(v1.y - v2.y, 2) + pow(v1.z - v2.z, 2));
}


// Vertex shader output structure
struct VS_OUTPUT
{
	float4 Position   : POSITION;   // vertex position
	float2 TexCoord0  : TEXCOORD0;  // tex coords // texture de base

	float3 TexCoord1  : TEXCOORD1;  // tex coords // texture de la lumire ambiante

	// lumires
	float3 TexCoord2  : TEXCOORD2;
	float3 TexCoord3  : TEXCOORD3;
	float3 TexCoord4  : TEXCOORD4;
	float3 TexCoord5  : TEXCOORD5;
	float3 TexCoord6  : TEXCOORD6;

	float4 realColor  : COLOR0;
	float4 colorFog   : COLOR1;
};


VS_OUTPUT vertexMain( in float4 vPosition : POSITION,
                      in float3 vNormal   : NORMAL,
                      float2 texCoord0    : TEXCOORD0)
{
	VS_OUTPUT Output;

	// les coordonnes de la texture de base sont inchangs
	Output.TexCoord0.xy = texCoord0;

	// transform position to clip space
	Output.Position = mul(vPosition, mWorldViewProj);

	// position du vertex dans le monde
	float3 worldpos = mul(vPosition, mWorld);

	// transform normal
	float4 normal = mul(vNormal, mInvWorld);
    normal = normalize(normal);

	Output.TexCoord1.x = 0.01 + max(dot(normal, - normalize(mSunDir)), 0)*0.98;


    // prparation des variables pour les lumires

    float3 light;
    float dist;
    float largeur;
    float fact;

    // ----------------------------------------------------------------------------
	// calcule le vecteur direction de la lumire 1

	light = worldpos - mLight1Pos;

    // volution dans l'intervalle de 1  0
    fact = 1 - smoothstep(mLight1Prop.x, mLight1Prop.y, length(light));

    Output.TexCoord2.x = 0.01 + max(dot(normal, - normalize(light)), 0)*0.98*fact;

    // ----------------------------------------------------------------------------
	// calcule le vecteur direction de la lumire 2

	light = worldpos - mLight2Pos;

    // volution dans l'intervalle de 1  0
    fact = 1 - smoothstep(mLight2Prop.x, mLight2Prop.y, length(light));

    Output.TexCoord3.x = 0.01 + max(dot(normal, - normalize(light)), 0)*0.98*fact;

    // ----------------------------------------------------------------------------
	// calcule le vecteur direction de la lumire 3

	light = worldpos - mLight3Pos;

    // volution dans l'intervalle de 1  0
    fact = 1 - smoothstep(mLight3Prop.x, mLight3Prop.y, length(light));

    Output.TexCoord4.x = 0.01 + max(dot(normal, - normalize(light)), 0)*0.98*fact;

    // ----------------------------------------------------------------------------
	// calcule le vecteur direction de la lumire 4

	light = worldpos - mLight4Pos;

    // volution dans l'intervalle de 1  0
    fact = 1 - smoothstep(mLight4Prop.x, mLight4Prop.y, length(light));

    Output.TexCoord5.x = 0.01 + max(dot(normal, - normalize(light)), 0)*0.98*fact;

    // ----------------------------------------------------------------------------
	// calcule le vecteur direction de la lumire 5

	light = worldpos - mLight5Pos;

    // volution dans l'intervalle de 1  0
    fact = 1 - smoothstep(mLight5Prop.x, mLight5Prop.y, length(light));

    Output.TexCoord6.x = 0.01 + max(dot(normal, - normalize(light)), 0)*0.98*fact;


    // ----------------------------------------------------------------------------
    // ----------------------------------------------------------------------------
	// Gestion de la lumire relle

	light = worldpos - mRealLight;

	// volution dans l'intervalle de 1  0
    fact = 1 - smoothstep(mRealLightProp.x, mRealLightProp.y, length(light.xyz));
    // fact = fact * max(dot(normal, -normalize(light)), 0);

    //Output.realColor = fact * mRealLightColor;

    // calculate light color
	float3 tmp = dot(- light, normal);
	tmp = lit(tmp.x, tmp.y, 1.0);

	tmp = mRealLightColor * tmp.y;
	Output.realColor = fact * float4(tmp.x, tmp.y, tmp.z, 0);

    // on enregistre la quantit d'effet fog
    Output.colorFog.x = smoothstep(fogDist.x, fogDist.y, length(camPos - worldpos.xyz));


/*
    // ----------------------------------------------------------------------------
	// calcule le vecteur direction de la lumire 2
	light = mLight2Pos;
	light = normalize(mul(light, mWorldViewProj) - Output.Position);

	dist = calcDist(Output.Position, mLight2Pos);

    // largeur de l'intervalle;
    largeur = mLight2Prop.y - mLight2Prop.x;

    // volution dans l'intervalle de 1  0
    fact = 1 - (dist - mLight2Prop.x)/largeur;
    fact = max(min(1, fact), 0);

    Output.TexCoord3.x = 0.01 + max(dot(normal, light), 0)*0.98*fact;

*/

    return Output;



	//Output.Diffuse = float4(1 - Output.TexCoord1.x/100, 0, 0, 0);

	// renormalize normal
	//normal = normalize(normal);

	// position in world coodinates
	//float3 worldpos = mul(mTransWorld, vPosition);

	// calculate light vector, vtxpos - lightpos
	//float3 lightVector = worldpos - mLightPos;

	// normalize light vector
	//lightVector = normalize(lightVector);

	// calculate light color
	//float3 tmp = dot(-lightVector, normal);

	//tmp = mLightColor * tmp.y;
	//Output.Diffuse = float4(tmp.x, tmp.y, tmp.z, 0);
	//Output.TexCoord0 = texCoord0;


}
/*
struct VS_OUTPUT_INK
{
	float4 Position   : POSITION;   // vertex position
	float4 Color      : COLOR0;
};


VS_OUTPUT_INK vertexInk( in float4 vPosition : POSITION,
                         in float3 vNormal   : NORMAL)
{
	VS_OUTPUT_INK Output;

    // INK

    //dcl_position v0
    //dcl_normal v3
    //dcl_texcoord v7

    //float4 r0 = vPosition;

    float3 contour = epaisseur * vNormal;

    vPosition.xyz = vPosition.xyz + contour.xyz

    //Output.Position = mul(vPosition, mWorldViewProj);

    //Output.Color = mWorldViewProj;
    //vPosition = mWorldViewProj;

	//return Output;
}
*/


// Pixel shader output structure
struct PS_OUTPUT
{
    float4 RGBColor : COLOR0;  // Pixel color
};


sampler2D tex0;
sampler2D tex1;
sampler2D tex2;

float4 mSunColor;
float3 mLight1Color = (0, 0, 0, 0);
float3 mLight2Color = (0, 0, 0, 0);
float3 mLight3Color = (0, 0, 0, 0);
float3 mLight4Color = (0, 0, 0, 0);
float3 mLight5Color = (0, 0, 0, 0);

// fog / brouillare
float4 fogColor;


float4 max4(float4 v1, float4 v2)
{
    float4 res;

    res.x = max(v1.x, v2.x);
    res.y = max(v1.y, v2.y);
    res.z = max(v1.z, v2.z);
    res.w = max(v1.w, v2.w);

    return res;
}


float3 maxColor(float3 c1, float3 c2)
{
    float s1 = c1.x + c1.y + c1.z;
    float s2 = c2.x + c2.y + c2.z;

    if(s1 > s2)
        return c1;
    else
        return c2;
}

PS_OUTPUT pixelMain( float2 TexCoord0 : TEXCOORD0,
                     float2 TexCoord1 : TEXCOORD1,
                     float2 TexCoord2 : TEXCOORD2,
                     float2 TexCoord3 : TEXCOORD3,
                     float2 TexCoord4 : TEXCOORD4,
                     float2 TexCoord5 : TEXCOORD5,
                     float2 TexCoord6 : TEXCOORD6,

                     float4 realColor : COLOR0,
                     float4 colorFog  : COLOR1)
{
	PS_OUTPUT Output;

	if(colorFog.x < 0.75)
	{

	float4 color       = tex2D(tex0, TexCoord0);
    float4 sunShade    = tex2D(tex1, TexCoord1)*mSunColor;

    // dtermine la couleur qui eclaire le plus la zone

    float3 best = sunShade.xyz;

    if(any(mLight1Color))
    {
        float3 light1Shade = tex2D(tex2, TexCoord2)*mLight1Color;
        best = maxColor(best, light1Shade);
    }

    if(any(mLight2Color))
    {
        float3 light2Shade = tex2D(tex2, TexCoord3)*mLight2Color;
        best = maxColor(best, light2Shade);
    }

    if(any(mLight3Color))
    {
        float3 light3Shade = tex2D(tex2, TexCoord4)*mLight3Color;
        best = maxColor(best,  light3Shade);
    }

    if(any(mLight4Color))
    {
        float3 light4Shade = tex2D(tex2, TexCoord5)*mLight4Color;
        best = maxColor(best,  light4Shade);
    }

    if(any(mLight5Color))
    {
        float3 light5Shade = tex2D(tex2, TexCoord6)*mLight5Color;
        best = maxColor(best,  light5Shade);
    }


    float fact = 1;

    if(colorFog.x > 0.25)
        fact = 0.6;


    float4 shade = sunShade;
    shade.x = min(best.x * color.x, 1) * fact;
    shade.y = min(best.y * color.y, 1) * fact;
    shade.z = min(best.y * color.z, 1) * fact;

	Output.RGBColor = shade * fact + fogColor * (1 - fact);

	}
	else
	{
        Output.RGBColor = fogColor;
	}

	return Output;
}
